@@ -54,9 +54,12 @@
 #include "gal/util/e-text-event-processor-emacs-like.h"
 #include "gal/util/e-i18n.h"
 #include "e-table-tooltip.h"
+#include "gal/a11y/e-table/gal-a11y-e-cell-registry.h"
+#include "gal/a11y/e-table/gal-a11y-e-cell-text.h"
 
 #define d(x)
 #define DO_SELECTION 1
+#define VIEW_TO_CELL(view) E_CELL_TEXT (((ECellView *)view)->ecell)
 
 #if d(!)0
 #define e_table_item_leave_edit_(x) (e_table_item_leave_edit((x)), g_print ("%s: e_table_item_leave_edit\n", __FUNCTION__))
@@ -92,6 +95,15 @@ enum {
 	E_SELECTION_CLIPBOARD
 };
 
+/* signals */
+enum {
+	TEXT_INSERTED,
+	TEXT_DELETED,
+	LAST_SIGNAL
+};
+
+static guint signals [LAST_SIGNAL] = { 0 };
+
 static GdkAtom clipboard_atom = GDK_NONE;
 
 #define PARENT_TYPE e_cell_get_type ()
@@ -492,8 +504,6 @@ build_attr_list (ECellTextView *text_view, int row, int text_length)
 static PangoLayout *
 layout_with_preedit (ECellTextView *text_view, int row, const char *text, gint width)
 {
-	ECellView *ecell_view = (ECellView *) text_view;
-	ECellText *ect = E_CELL_TEXT (ecell_view->ecell);
 	CellEdit *edit = text_view->edit;
 	PangoAttrList *attrs ;
 	PangoLayout *layout;
@@ -557,7 +567,6 @@ build_layout (ECellTextView *text_view, int row, const char *text, gint width)
 	ECellText *ect = E_CELL_TEXT (ecell_view->ecell);
 	PangoAttrList *attrs ;
 	PangoLayout *layout;
-	gboolean bold, strikeout, underline;
 
 	layout = gtk_widget_create_pango_layout (GTK_WIDGET (((GnomeCanvasItem *)ecell_view->e_table_item_view)->canvas), text);
 
@@ -874,7 +883,6 @@ static gint
 ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col, int row, ECellFlags flags, ECellActions *actions)
 {
 	ECellTextView *text_view = (ECellTextView *) ecell_view;
-	ECellText *ect = (ECellText *) ((ECellView *)text_view)->ecell;
 	ETextEventProcessorEvent e_tep_event;
 	gboolean edit_display = FALSE;
 	gint preedit_len;
@@ -1302,6 +1310,9 @@ ect_print (ECellView *ecell_view, GnomePrintContext *context,
 	GnomeFont *font = get_font_for_size (16);
 	char *string;
 	ECellText *ect = E_CELL_TEXT(ecell_view->ecell);
+	double ty, ly, text_width;
+	gboolean strikeout, underline;
+
 	string = e_cell_text_get_text(ect, ecell_view->e_table_model, model_col, row);
 	gnome_print_gsave(context);
 	if (gnome_print_moveto(context, 2, 2) == -1)
@@ -1316,8 +1327,34 @@ ect_print (ECellView *ecell_view, GnomePrintContext *context,
 				/* FIXME */;
 	if (gnome_print_clip(context) == -1)
 				/* FIXME */;
-	gnome_print_moveto(context, 2, (height - gnome_font_get_ascender(font) - gnome_font_get_descender(font)) / 2);
 
+	ty = (height - gnome_font_get_ascender(font) - gnome_font_get_descender(font)) / 2;
+	text_width = gnome_font_get_width_utf8 (font, string);
+
+	strikeout = ect->strikeout_column >= 0 && row >= 0 &&
+		e_table_model_value_at (ecell_view->e_table_model, ect->strikeout_column, row);
+	underline = ect->underline_column >= 0 && row >= 0 &&
+		e_table_model_value_at(ecell_view->e_table_model, ect->underline_column, row);
+
+	if (underline) {
+		ly = ty + gnome_font_get_underline_position (font);
+		gnome_print_newpath (context);
+		gnome_print_moveto (context, 2, ly);
+		gnome_print_lineto (context, MIN (2 + text_width, width - 2), ly);
+		gnome_print_setlinewidth (context, gnome_font_get_underline_thickness (font));
+		gnome_print_stroke (context);
+	}
+
+	if (strikeout) {
+		ly = ty + (gnome_font_get_ascender (font)  - gnome_font_get_underline_thickness (font))/ 2.0;
+		gnome_print_newpath (context);
+		gnome_print_moveto (context, 2, ly);
+		gnome_print_lineto (context, MIN (2 + text_width, width - 2), ly);
+		gnome_print_setlinewidth (context, gnome_font_get_underline_thickness (font));
+		gnome_print_stroke (context);
+	}
+
+	gnome_print_moveto(context, 2, ty);
 	gnome_print_setfont(context, font);
 	gnome_print_show(context, string);
 	gnome_print_grestore(context);
@@ -1671,32 +1708,56 @@ e_cell_text_class_init (GObjectClass *object_class)
 
 	parent_class = g_type_class_ref (PARENT_TYPE);
 
+	signals [TEXT_INSERTED] = 
+		g_signal_new ("text_inserted",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (ECellTextClass, text_inserted),
+			      NULL, NULL,
+			      e_marshal_VOID__POINTER_INT_INT_INT_INT,
+			      G_TYPE_NONE, 5,
+			      G_TYPE_POINTER, G_TYPE_INT, G_TYPE_INT,
+			      G_TYPE_INT, G_TYPE_INT);
+
+	signals [TEXT_DELETED] = 
+		g_signal_new ("text_deleted",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_FIRST,
+			      G_STRUCT_OFFSET (ECellTextClass, text_deleted),
+			      NULL, NULL,
+			      e_marshal_VOID__POINTER_INT_INT_INT_INT,
+			      G_TYPE_NONE, 5,
+			      G_TYPE_POINTER, G_TYPE_INT, G_TYPE_INT,
+			      G_TYPE_INT, G_TYPE_INT);
+
+
+
 	g_object_class_install_property (object_class, PROP_STRIKEOUT_COLUMN,
 					 g_param_spec_int ("strikeout_column",
 							   _("Strikeout Column"),
 							   /*_( */"XXX blurb" /*)*/,
-							   0, G_MAXINT, 0,
+							   -1, G_MAXINT, -1,
 							   G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class, PROP_UNDERLINE_COLUMN,
 					 g_param_spec_int ("underline_column",
 							   _("Underline Column"),
 							   /*_( */"XXX blurb" /*)*/,
-							   0, G_MAXINT, 0,
+							   -1, G_MAXINT, -1,
 							   G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class, PROP_BOLD_COLUMN,
 					 g_param_spec_int ("bold_column",
 							   _("Bold Column"),
 							   /*_( */"XXX blurb" /*)*/,
-							   0, G_MAXINT, 0,
+							   -1, G_MAXINT, -1,
 							   G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class, PROP_COLOR_COLUMN,
 					 g_param_spec_int ("color_column",
 							   _("Color Column"),
 							   /*_( */"XXX blurb" /*)*/,
-							   0, G_MAXINT, 0,
+							   -1, G_MAXINT, -1,
 							   G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class, PROP_EDITABLE,
@@ -1710,7 +1771,7 @@ e_cell_text_class_init (GObjectClass *object_class)
 					 g_param_spec_int ("bg_color_column",
 							   _("BG Color Column"),
 							   /*_( */"XXX blurb" /*)*/,
-							   0, G_MAXINT, 0,
+							   -1, G_MAXINT, -1,
 							   G_PARAM_READWRITE));
 
 	if (!clipboard_atom)
@@ -1724,6 +1785,8 @@ e_cell_text_class_init (GObjectClass *object_class)
 			use_ellipsis_default = FALSE;
 		}
 	}
+	
+	gal_a11y_e_cell_registry_add_cell_type (NULL, E_CELL_TEXT_TYPE, gal_a11y_e_cell_text_new);
 }
 
 
@@ -1751,7 +1814,6 @@ e_cell_text_commit_cb (GtkIMContext *context,
 		  ECellTextView    *tv)
 {
 	CellEdit *edit = tv->edit;
-	ECellText *ect = (ECellText *) ((ECellView *)tv)->ecell;
 	ETextEventProcessorCommand command;
 	                                                  
 	if (g_utf8_validate (str, strlen (str), NULL)) {
@@ -2154,6 +2216,8 @@ _delete_selection (ECellTextView *text_view)
 	memmove (sp, ep, length);
 
 	edit->selection_end = edit->selection_start;
+
+	g_signal_emit (VIEW_TO_CELL (text_view), signals[TEXT_DELETED], 0, text_view, edit->selection_start, ep-sp, edit->row, edit->model_col);
 }
 
 /* fixme: */
@@ -2181,6 +2245,8 @@ _insert (ECellTextView *text_view, char *string, int value)
 
 	edit->selection_start += value;
 	edit->selection_end = edit->selection_start;
+
+	g_signal_emit (VIEW_TO_CELL (text_view), signals[TEXT_INSERTED], 0, text_view, edit->selection_end-value, value, edit->row, edit->model_col);
 }
 
 static void
@@ -2570,3 +2636,219 @@ e_cell_text_get_color (ECellTextView *cell_view, gchar *color_spec)
 	return color;
 }
 
+/**
+ * e_cell_text_set_selection:
+ * @cell_view: the given cell view
+ * @col: column of the given cell in the view
+ * @row: row of the given cell in the view
+ * @start: start offset of the selection
+ * @end: end offset of the selection
+ *
+ * Sets the selection of given text cell.
+ * If the current editing cell is not the given cell, this function
+ * will return FALSE;
+ *
+ * If success, the [start, end) part of the text will be selected.
+ *
+ * This API is most likely to be used by a11y implementations.
+ * 
+ * Returns: whether the action is successful.
+ */
+gboolean
+e_cell_text_set_selection (ECellView *cell_view,
+			   gint col,
+			   gint row,
+			   gint start,
+			   gint end)
+{
+	ECellTextView *ectv;
+	CellEdit *edit;
+	ETextEventProcessorCommand command1, command2;
+
+	ectv = (ECellTextView *)cell_view;
+	edit = ectv->edit;
+	if (!edit)
+		return FALSE;
+
+	if (edit->view_col != col || edit->row != row)
+		return FALSE;
+
+	command1.action = E_TEP_MOVE;
+	command1.position = E_TEP_VALUE;
+	command1.value = start;
+	e_cell_text_view_command (edit->tep, &command1, edit);
+
+	command2.action = E_TEP_SELECT;
+	command2.position = E_TEP_VALUE;
+	command2.value = end;
+	e_cell_text_view_command (edit->tep, &command2, edit);
+
+	return TRUE;
+}
+
+/**
+ * e_cell_text_get_selection:
+ * @cell_view: the given cell view
+ * @col: column of the given cell in the view
+ * @row: row of the given cell in the view
+ * @start: a pointer to an int value indicates the start offset of the selection
+ * @end: a pointer to an int value indicates the end offset of the selection
+ *
+ * Gets the selection of given text cell.
+ * If the current editing cell is not the given cell, this function
+ * will return FALSE;
+ *
+ * This API is most likely to be used by a11y implementations.
+ * 
+ * Returns: whether the action is successful.
+ */
+gboolean
+e_cell_text_get_selection (ECellView *cell_view,
+			   gint col,
+			   gint row,
+			   gint *start,
+			   gint *end)
+{
+	ECellTextView *ectv;
+	CellEdit *edit;
+
+	ectv = (ECellTextView *)cell_view;
+	edit = ectv->edit;
+	if (!edit)
+		return FALSE;
+
+	if (edit->view_col != col || edit->row != row)
+		return FALSE;
+
+	if (start)
+		*start = edit->selection_start;
+	if (end)
+		*end = edit->selection_end;
+	return TRUE;
+}
+
+/**
+ * e_cell_text_copy_clipboard:
+ * @cell_view: the given cell view
+ * @col: column of the given cell in the view
+ * @row: row of the given cell in the view
+ *
+ * Copys the selected text to clipboard.
+ *
+ * This API is most likely to be used by a11y implementations.
+ */
+void
+e_cell_text_copy_clipboard (ECellView *cell_view, gint col, gint row)
+{
+	ECellTextView *ectv;
+	CellEdit *edit;
+	ETextEventProcessorCommand command;
+
+	ectv = (ECellTextView *)cell_view;
+	edit = ectv->edit;
+	if (!edit)
+		return;
+
+	if (edit->view_col != col || edit->row != row)
+		return;
+
+	command.action = E_TEP_COPY;
+	command.time = GDK_CURRENT_TIME;
+	e_cell_text_view_command (edit->tep, &command, edit);
+}
+
+/**
+ * e_cell_text_paste_clipboard:
+ * @cell_view: the given cell view
+ * @col: column of the given cell in the view
+ * @row: row of the given cell in the view
+ *
+ * Pastes the text from the clipboardt.
+ *
+ * This API is most likely to be used by a11y implementations.
+ */
+void
+e_cell_text_paste_clipboard (ECellView *cell_view, gint col, gint row)
+{
+	ECellTextView *ectv;
+	CellEdit *edit;
+	ETextEventProcessorCommand command;
+
+	ectv = (ECellTextView *)cell_view;
+	edit = ectv->edit;
+	if (!edit)
+		return;
+
+	if (edit->view_col != col || edit->row != row)
+		return;
+
+	command.action = E_TEP_PASTE;
+	command.time = GDK_CURRENT_TIME;
+	e_cell_text_view_command (edit->tep, &command, edit);
+}
+
+/**
+ * e_cell_text_delete_selection:
+ * @cell_view: the given cell view
+ * @col: column of the given cell in the view
+ * @row: row of the given cell in the view
+ *
+ * Deletes the selected text of the cell.
+ *
+ * This API is most likely to be used by a11y implementations.
+ */
+void
+e_cell_text_delete_selection (ECellView *cell_view, gint col, gint row)
+{
+	ECellTextView *ectv;
+	CellEdit *edit;
+	ETextEventProcessorCommand command;
+
+	ectv = (ECellTextView *)cell_view;
+	edit = ectv->edit;
+	if (!edit)
+		return;
+
+	if (edit->view_col != col || edit->row != row)
+		return;
+
+	command.action = E_TEP_DELETE;
+	command.position = E_TEP_SELECTION;
+	e_cell_text_view_command (edit->tep, &command, edit);
+}
+
+/**
+ * e_cell_text_get_text_by_view:
+ * @cell_view: the given cell view
+ * @col: column of the given cell in the model
+ * @row: row of the given cell in the model
+ * 
+ * Get the cell's text directly from CellEdit,
+ * during editting this cell, the cell's text value maybe inconsistant
+ * with the text got from table_model.
+ * The caller should free the text after using it.
+ *
+ * This API is most likely to be used by a11y implementations.
+ */
+char *
+e_cell_text_get_text_by_view (ECellView *cell_view, gint col, gint row)
+{
+	ECellTextView *ectv;
+	CellEdit *edit;
+	gchar	*ret, *model_text;
+
+	ectv = (ECellTextView *)cell_view;
+	edit = ectv->edit;
+	
+	if (edit && ectv->edit->row == row && ectv->edit->model_col == col) { /* being editted now */
+		ret = g_strdup (edit->text);
+	} else{
+		model_text = e_cell_text_get_text (E_CELL_TEXT (cell_view->ecell), 
+					     cell_view->e_table_model, col, row);
+		ret = g_strdup (model_text);
+		e_cell_text_free_text (E_CELL_TEXT (cell_view->ecell), model_text);
+	}
+
+	return ret;
+
+}
